/*
 *  obdictionary.h
 *
 *  Copyright 2006 Cycling '74. All rights reserved.
 *
 */

#ifndef __OBDICTIONARY_H__
#define __OBDICTIONARY_H__

#ifdef __cplusplus
	extern "C" {
#endif // __cplusplus

#if C74_PRAGMA_STRUCT_PACKPUSH
    #pragma pack(push, 2)
#elif C74_PRAGMA_STRUCT_PACK
    #pragma pack(2)
#endif

// these structs are provided for debugging convenience, but 
// are subject to change, and should not be modified directly

typedef struct _dictionary_entry {
	t_object		e_obj;
	t_symbol		*e_key;			// redundant with hashtab, but allows getting key during linklist traversal
	t_atom			e_value; 
} t_dictionary_entry; 

typedef struct _dictionary
{
	t_object		d_obj;
	t_hashtab		*d_hashtab; 
	t_linklist		*d_linklist; 
} t_dictionary; 

#if C74_PRAGMA_STRUCT_PACKPUSH
    #pragma pack(pop)
#elif C74_PRAGMA_STRUCT_PACK
    #pragma pack()
#endif

t_dictionary* dictionary_new();
t_dictionary* dictionary_prototypefromclass(t_class *c);
t_max_err dictionary_appendlong(t_dictionary *d, t_symbol *key, long value);
t_max_err dictionary_appendfloat(t_dictionary *d, t_symbol *key, double value); 
t_max_err dictionary_appendsym(t_dictionary *d, t_symbol *key, t_symbol *value); 
t_max_err dictionary_appendatom(t_dictionary *d, t_symbol *key, t_atom *value);
t_max_err dictionary_appendattribute(t_dictionary *d, t_symbol *key, t_symbol *attrname, t_object *obj);  
t_max_err dictionary_appendstring(t_dictionary *d, t_symbol *key, const char *value); 
t_max_err dictionary_appendatoms(t_dictionary *d, t_symbol *key, long argc, t_atom *argv); 
t_max_err dictionary_appendatomarray(t_dictionary *d, t_symbol *key, t_object *value); 
t_max_err dictionary_appenddictionary(t_dictionary *d, t_symbol *key, t_object *value); 
t_max_err dictionary_appendobject(t_dictionary *d, t_symbol *key, t_object *value); 
t_max_err dictionary_appendobject_flags(t_dictionary *d, t_symbol *key, t_object *value, long flags); 
t_max_err dictionary_appendbinbuf(t_dictionary *d, t_symbol *key, void *value); 
t_max_err dictionary_getlong(t_dictionary *d, t_symbol *key, long *value);
t_max_err dictionary_getfloat(t_dictionary *d, t_symbol *key, double *value); 
t_max_err dictionary_getsym(t_dictionary *d, t_symbol *key, t_symbol **value); 
t_max_err dictionary_getatom(t_dictionary *d, t_symbol *key, t_atom *value); 
t_max_err dictionary_getattribute(t_dictionary *d, t_symbol *key, t_symbol *attrname, t_object *obj);	
t_max_err dictionary_getstring(t_dictionary *d, t_symbol *key, const char **value); 
t_max_err dictionary_getatoms(t_dictionary *d, t_symbol *key, long *argc, t_atom **argv);   // does not make a copy
t_max_err dictionary_copyatoms(t_dictionary *d, t_symbol *key, long *argc, t_atom **argv);  // does make a copy
t_max_err dictionary_getatomarray(t_dictionary *d, t_symbol *key, t_object **value); 
t_max_err dictionary_getdictionary(t_dictionary *d, t_symbol *key, t_object **value); 
t_max_err dictionary_getobject(t_dictionary *d, t_symbol *key, t_object **value); 
long dictionary_entryisstring(t_dictionary *d, t_symbol *key); 
long dictionary_entryisatomarray(t_dictionary *d, t_symbol *key);
long dictionary_entryisdictionary(t_dictionary *d, t_symbol *key); 
long dictionary_hasentry(t_dictionary *d, t_symbol *key); 
long dictionary_getentrycount(t_dictionary *d);
t_max_err dictionary_getkeys(t_dictionary *d, long *numkeys, t_symbol ***keys);   // free keys with freekeys function, not mm_freebytes
void dictionary_freekeys(t_dictionary *d, long numkeys, t_symbol **keys); 
t_max_err dictionary_deleteentry(t_dictionary *d, t_symbol *key);		// remove a value from the dictionary
t_max_err dictionary_chuckentry(t_dictionary *d, t_symbol *key);		// remove a value from the dictionary without deleting it
t_max_err dictionary_clear(t_dictionary *d);
// funall will pass the t_dictionary_entry pointer to the fun
// use the methods below to access the fields
void dictionary_funall(t_dictionary *x, method fun, void *arg);
t_symbol* dictionary_entry_getkey(t_dictionary_entry *x);
void dictionary_entry_getvalue(t_dictionary_entry *x, t_atom *value); 
t_max_err dictionary_copyunique(t_dictionary *d, t_dictionary *copyfrom);

t_max_err dictionary_getdeflong(t_dictionary *d, t_symbol *s, long *value, long def);
t_max_err dictionary_getdeffloat(t_dictionary *d, t_symbol *key, double *value, double def);
t_max_err dictionary_getdefsym(t_dictionary *d, t_symbol *key, t_symbol **value, t_symbol *def);
t_max_err dictionary_getdefatom(t_dictionary *d, t_symbol *key, t_atom *value, t_atom *def);
t_max_err dictionary_getdefstring(t_dictionary *d, t_symbol *key, const char **value, char *def);
t_max_err dictionary_getdefatoms(t_dictionary *d, t_symbol *key, long *argc, t_atom **argv, t_atom *def);
t_max_err dictionary_copydefatoms(t_dictionary *d, t_symbol *key, long *argc, t_atom **argv, t_atom *def);

t_max_err dictionary_dump(t_dictionary *d, long recurse, long console);
t_max_err dictionary_copyentries(t_dictionary *src, t_dictionary *dst, t_symbol **keys);

t_dictionary *dictionary_sprintf(char *fmt, ...);
t_object *newobject_sprintf(t_object *patcher, char *fmt, ...);
t_object *newobject_fromdictionary(t_object *patcher, t_dictionary *d);


#ifdef __cplusplus
}
#endif // __cplusplus

#endif //__OBDICTIONARY_H__
